@@ -3,6 +3,10 @@
 
 #include "global.h"
 
+// For some reason the symbian MOC doesn't like it if I don't include QObject
+// even though it is present in QtCore which is included in global.h
+#include <QObject>
+
 class ContactsXmlHandler : public QObject, public QXmlDefaultHandler
 {
     Q_OBJECT
@@ -14,6 +18,8 @@ public:
     quint32 getTotalContacts ();
     quint32 getUsableContacts ();
 
+    void setEmitLog (bool enable = true);
+
 signals:
     void status (const QString &text, int timeout = 2000);
 
@@ -42,6 +48,8 @@ protected:
 
     PhoneInfo   currPhone;
     ContactInfo currInfo;
+
+    bool        bEmitLog;
 };
 
 #endif //__CONTACTSXMLHANDLER_H__
